!*****************************************************************************************
!     NEMURO model     Jun 13, 2002  written by Yasuhiro Yamanaka
!
!     Version 1.0 parameterized to Station A7
!
!     Distributed 16December03
!
! Original code sent by Kishi for Nemuro 2002 workshop with A7 parameters
!                                                     1/14/02 bam
!*****************************************************************************************
      program NEMURO
      implicit none
!     ..... Control for Time Ingration .....
      character(19)     :: Cstart = '1991/01/01 00:00:00'  ! Starting date
      character(19)     :: Cend   = '1994/01/01 00:00:00'  ! Ending date
      character(19)     :: Cstep  = '0000/00/00 01:00:00'  ! Time step 
      character(19)     :: Cmon   = '0000/00/01 00:00:00'  ! Monitor Interval
      character(19)     :: CTime
      real(8)           :: dt, TTime, Tbefore, Season, Tmon
      integer           :: Iyr, Imon, Iday, Ihour, Imin, Isec
!     ..... scale conversion .....
      real(8),parameter :: d2s       = 86400.0d0    ! day ---> sec
      real(8),parameter :: mcr       = 1.0d-6       ! micro
!     ..... Prognostic Variables (with initial conditions) and Thier Source Term .....
      real(8)           :: TPS    =  0.1D-6, QPS    ! Small Phytoplankton [molN/l]
      real(8)           :: TPL    =  0.1D-6, QPL    ! Large Phytoplankton
      real(8)           :: TZS    =  0.1D-6, QZS    ! Small Zooplankton
      real(8)           :: TZL    =  0.1D-6, QZL    ! Large Zooplankton
      real(8)           :: TZP    =  0.1D-6, QZP    ! Pradatory Zooplankton
      real(8)           :: TNO3   =  5.0D-6, QNO3   ! Nitrate
      real(8)           :: TNH4   =  0.1D-6, QNH4   ! Ammmonium
      real(8)           :: TPON   =  0.1D-6, QPON   ! Particulate Organic Nitrogen
      real(8)           :: TDON   =  0.1D-6, QDON   ! dissolved Organic Nitrogen
      real(8)           :: TSiOH4 = 10.0D-6, QSiOH4 ! Silicate
      real(8)           :: TOpal  =  0.1D-7, QOpal  ! Particulate Opal
!     ...... Light Condition Parameters .......
      real(8),parameter :: alpha1   = 4.0D-2     ! Light Dissipation coefficient of sea water[/m]
      real(8),parameter :: alpha2   = 4.0D4      ! PS+PL Selfshading coefficientS+PL      [l/molN/m]
      real(8),parameter :: IoptS    = 0.15D0     ! PS Optimum Light Intensity  S          [ly/min]
      real(8),parameter :: IoptL    = 0.15D0     ! PL Optimum Light Intensity             [ly/min]
      integer,parameter :: LLN      = 10         ! Number of sublayer for calculating of Lfc
      real(8)           :: LfcS                  ! Light factor for PS
      real(8)           :: LfcL                  ! Light factor for PL
      real(8)           :: kappa, Lint, dLint, LfcUS, LfcUL, LfcDS, LfcDL
      integer           :: L
!     ...... biological Parameters ......
      real(8),parameter :: VmaxS    = 0.4D0/d2s  ! PS Maximum Photosynthetic rate @0degC   [/s]
      real(8),parameter :: KNO3S    = 1.0D-6     ! PS Half satuation constant for Nitrate  [molN/l]
      real(8),parameter :: KNH4S    = 0.1D-6     ! PS Half satuation constant for Ammonium [molN/l]
      real(8),parameter :: PusaiS   = 1.5D6      ! PS Ammonium Inhibition Coefficient      [l/molN]
      real(8),parameter :: KGppS    = 6.93D-2    ! PS Temp. Coeff. for Photosynthetic Rate [/degC]
      real(8),parameter :: MorPS0   = 5.85D4/d2s ! PS Mortality Rate @0degC                [/s]
      real(8),parameter :: KMorPS   = 6.93D-2    ! PS Temp. Coeff. for Mortality           [/degC]
      real(8),parameter :: ResPS0   = 0.03D0/d2s ! PS Respiration Rate at @0degC           [/s]
      real(8),parameter :: KResPS   = 0.0519D0   ! PS Temp. Coeff. for Respiration         [/degC]
      real(8),parameter :: GammaS   = 0.135D0    ! PS Ratio of Extracell. Excret. to Photo.[(nodim)]
      real(8),parameter :: VmaxL    = 0.8D0/d2s  ! PL Maximum Photosynthetic rate @0degC   [/s]
      real(8),parameter :: KNO3L    = 3.00D-6    ! PL Half satuation constant for Nitrate  [molN/l]
      real(8),parameter :: KNH4L    = 0.30D-6    ! PL Half satuation constant for Ammonium [molN/l]
      real(8),parameter :: KSiL     = 6.00D-6    ! PL Half satuation constant for Silicate [molSi/l]
      real(8),parameter :: PusaiL   = 1.50D6     ! PL Ammonium Inhibition Coefficient      [l/molN]
      real(8),parameter :: KGppL    = 6.93D-2    ! PL Temp. Coeff. for Photosynthetic Rate [/degC]
      real(8),parameter :: MorPL0   = 2.90D4/d2s ! PL Mortality Rate @0degC                [/s]
      real(8),parameter :: KMorPL   = 6.93D-2    ! PL Temp. Coeff. for Mortality           [/degC]
      real(8),parameter :: ResPL0   = 0.03D0/d2s ! PL Respiration Rate at @0degC           [/s]
      real(8),parameter :: KResPL   = 0.0519D0   ! PL Temp. Coeff. for Respiration         [/degC]
      real(8),parameter :: GammaL   = 0.135D0    ! PL Ratio of Extracell. Excret. to Photo.[(nodim)]
      real(8),parameter :: GRmaxS   = 0.40D0/d2s ! ZS Maximum Rate of Grazing PS @0degC    [/s]
      real(8),parameter :: KGraS    = 6.93D-2    ! ZS Temp. Coeff. for Grazing             [/degC]
      real(8),parameter :: LamS     = 1.40D6     ! ZS Ivlev constant                       [l/molN]
      real(8),parameter :: PS2ZSstar= 0.043D-6   ! ZS Threshold Value for Grazing PS       [molN/l]
      real(8),parameter :: AlphaZS  = 0.70D0     ! ZS Assimilation Efficiency              [(nodim)]
      real(8),parameter :: BetaZS   = 0.30D0     ! ZS Growth Efficiency                    [(nodim)]
      real(8),parameter :: MorZS0   = 5.85D4/d2s ! ZS Mortality Rate @0degC                [/s]
      real(8),parameter :: KMorZS   = 0.0693D0   ! ZS Temp. Coeff. for Mortality           [/degC]
      real(8),parameter :: GRmaxLps = 0.10D0/d2s ! ZL Maximum Rate of Grazing PS @0degC    [/s]
      real(8),parameter :: GRmaxLpl = 0.40D0/d2s ! ZL Maximum Rate of Grazing PL @0degC    [/s]
      real(8),parameter :: GRmaxLzs = 0.40D0/d2s ! ZL Maximum Rate of Grazing ZS @0degC    [/s]
      real(8),parameter :: KGraL    = 6.93D-2    ! ZL Temp. Coeff. for Grazing             [/degC]
      real(8),parameter :: LamL     = 1.4000D6   ! ZL Ivlev constant                       [l/molN]
      real(8),parameter :: PS2ZLstar= 4.00D-8    ! ZL Threshold Value for Grazing PS       [molN/l]
      real(8),parameter :: PL2ZLstar= 4.00D-8    ! ZL Threshold Value for Grazing PL       [molN/l]
      real(8),parameter :: ZS2ZLstar= 4.00D-8    ! ZL Threshold Value for Grazing ZS       [molN/l]
      real(8),parameter :: AlphaZL  = 0.70D0     ! ZL Assimilation Efficiency              [(nodim)]
      real(8),parameter :: BetaZL   = 0.30D0     ! ZL Growth Efficiency                    [(nodim)]
      real(8),parameter :: MorZL0   = 5.85D4/d2s ! ZL Mortality Rate @0degC                [/s]
      real(8),parameter :: KMorZL   = 0.0693D0   ! ZL Temp. Coeff. for Mortality           [/degC]
      real(8),parameter :: GRmaxPpl = 0.20D0/d2s ! ZP Maximum rate of grazing PL @0degC    [/s]
      real(8),parameter :: GRmaxPzs = 0.20D0/d2s ! ZP Maximum rate of grazing ZS @0degC    [/s]
      real(8),parameter :: GRmaxPzl = 0.20D0/d2s ! ZP Maximum rate of grazing ZL @0degC    [/s]
      real(8),parameter :: KGraP    = 6.93D-2    ! ZP Temp. Coeff. for grazing             [/degC]
      real(8),parameter :: LamP     = 1.4000D6   ! ZP Ivlev constant                       [l/molN]
      real(8),parameter :: PL2ZPstar= 4.00D-8    ! ZP Threshold Value for Grazing PL       [molN/l]
      real(8),parameter :: ZS2ZPstar= 4.00D-8    ! ZP Threshold Value for Grazing ZS       [molN/l]
      real(8),parameter :: ZL2ZPstar= 4.00D-8    ! ZP Threshold Value for Grazing ZL       [molN/l]
      real(8),parameter :: PusaiPL  = 4.605D6    ! ZP Preference Coeff. for PL             [l/molN]
      real(8),parameter :: PusaiZS  = 3.010D6    ! ZP Preference Coeff. for ZS             [l/molN]
      real(8),parameter :: AlphaZP  = 0.70D0     ! ZP Assimilation Efficiency              [(nodim)]
      real(8),parameter :: BetaZP   = 0.30D0     ! ZP Growth Efficiency                    [(nodim)]
      real(8),parameter :: MorZP0   = 5.85D4/d2s ! ZP Mortality Rate @0degC                [/s]
      real(8),parameter :: KMorZP   = 0.0693D0   ! ZP Temp. Coeff. for Mortality           [/degC]
      real(8),parameter :: Nit0     = 0.03D0/d2s ! NH4 Nitrification Rate @0degC           [/s]
      real(8),parameter :: KNit     = 0.0693D0   ! NH4 Temp. coefficient for Nitrification [/degC]
      real(8),parameter :: VP2N0    = 0.10D0/d2s ! PON Decomp. Rate to Ammonium @0degC     [/s]
      real(8),parameter :: KP2N     = 6.93D-2    ! PON Temp. Coeff. for Decomp. to Ammon.  [/degC]
      real(8),parameter :: VP2D0    = 0.10D0/d2s ! PON Decomp. Rate to DON @0degC          [/s]
      real(8),parameter :: KP2D     = 6.93D-2    ! PON Temp. Coeff. for Decomp. to DON     [/degC]
      real(8),parameter :: VD2N0    = 0.20D0/d2s ! DON Decomp. Rate to Ammonium @0degC     [/s]
      real(8),parameter :: KD2N     = 6.93D-2    ! DON Temp. Coeff. for Decomp. to Ammon.  [/degC]
      real(8),parameter :: VO2S0    = 0.10D0/d2s ! Opal Decomp. Rate to Silicate @0degC    [/s]
      real(8),parameter :: KO2S     = 6.93D-2    ! Opal Temp. Coeff. for Decomp.to Silicate[/degC]
      real(8),parameter :: RSiN     = 2.0D0          !Si/N ratio                      [molSi/molN]
      real(8),parameter :: RCN      = 106.0D0/16.0D0 !C/N ratio                        [molC/molN]
!     ..... bottom boundary Condition .....
      real(8),parameter :: setVPON  = 40.0D0/d2s ! Settling velocity of PON                [m/s]
      real(8),parameter :: setVOpal = 40.0D0/d2s ! Settling velocity of Opal               [m/s]
      real(8),parameter :: TNO3d   =  25.0d-6    ! Nitrate Concentraion in the Deep Layer  [molN/l]
      real(8),parameter :: TSiOH4d =  35.0d-6    ! Silicate Concentraion in the Deep Layer [molSi/l]
!     ...... Paramters of ZL Vertical Migration .....
      character(19)     :: CZup ='0000/04/01 00:00:00' ! Date Coming up to the Euphotic Layer
      character(19)     :: CZdwn='0000/09/01 00:00:00' ! Date Returning to the Deep Layer
      real(8)           :: TZup, TZdwn, TZLd, SVRate=0.2D0
      integer           :: IyrU, ImonU, IdayU ,IhourU, IminU, IsecU
      integer           :: IyrD, ImonD, IdayD ,IhourD, IminD, IsecD
!
      real(8)           :: GppPSn,  GppNPSn, GppAPSn, RnewS, ResPSn,  MorPSn,  ExcPSn
      real(8)           :: GppPLn,  GppNPLn, GppAPLn, RnewL, ResPLn,  MorPLn,  ExcPLn
      real(8)           :: GppPLsi, GppSiPLsi,               ResPLsi, MorPLsi, ExcPLsi
      real(8)           :: GraPS2ZSn, GraPS2ZLn,  GraPL2ZLn, GraPL2ZLsi, GraZS2ZLn
      real(8)           :: GraPL2ZPn, GraPL2ZPsi, GraZS2ZPn, GraZL2ZPn
      real(8)           :: EgeZSn, MorZSn, ExcZSn, EgeZLn, EgeZLsi, MorZLn, ExcZLn
      real(8)           ::                         EgeZPn, EgeZPsi, MorZPn, ExcZPn
      real(8)           :: DecP2N, DecP2D, DecD2N, DecO2S, Nit
      real(8)           :: ExpPON, ExpOpal,ExcNO3, ExcSiOH4
      integer           :: lt=0 , nt
!
!     ..... Environmental Condition .....
      real(8)           :: Temp                            ! Temperature [degC]
      real(8)           :: Lint0                           ! Light Intencity at sea surface [ly/min]
      real(8)           :: MLD = 30.0d0                    ! Mixed Layer Depth [m]
      real(8)           :: ExcTime = 1.0d0 / (100.0d0*d2s) ! Exch. Coeff. between Sur-Deep [/s]
!     ..... statement function & def. type of functions .....
      real(8)           :: cd2tt, nd2tt
      character(19)     :: tt2cd
      real(8)           :: Td, GraF, Mich, a, b, c
      Td  (a,b)   = a * exp(b*Temp)
      GraF(a,b,c) = MAX( 0.0D0, 1.0 - exp(a * (b - c)))
      Mich(a,b)   = b / ( a + b )
!
!     ***** Initial Setting *****
!     ..... for time control .....
      TTime = cd2tt(Cstart)                                ! Starting Date
      CTime = TT2CD(TTime)                                 ! present time (charactor form)
      dt    = cd2tt(Cstep) - cd2tt('0000/00/00 00:00:00')  ! Time Step (real8 form)
      Tmon  = cd2tt(Cmon)  - cd2tt('0000/00/00 00:00:00')  ! Monitor Interval (real8 form)
      nt = NINT( ( cd2tt(Cend) - cd2tt(Cstart) ) / dt )    ! Total Time Steps
!     ..... for Vertical Migration ......
      TZup  = CD2TT( CZup  )
      TZdwn = CD2TT( CZdwn )
      call TT2ND(IyrU, ImonU, IdayU ,IhourU, IminU, IsecU ,TZup )
      call TT2ND(IyrD, ImonD, IdayD ,IhourD, IminD, IsecD ,TZdwn)
      TZLd = TZL  ! ZL living in the deep layer at the initial condition
      TZL  = 0.0
!     ..... File Open for monitoring output .....
      open( 10, file='Results.csv', form='FORMATTED' )
      write(10,'(A,13(",", A))') 'Time(day)', &
                    'NO3'   , 'NH4'     , 'PS'     , 'PL'      , &
                    'ZS'    , 'ZL'      , 'ZP'     , 'PON'     , &
                    'DON'   , 'SiOH4'   , 'Opal'   , 'TotalN'  , 'TotalSi'
      write(10,'(A,11(",", F8.4))') CTime, &
                    TNO3/mcr, TNH4  /mcr, TPS  /mcr, TPL /mcr, &
                    TZS /mcr, TZL   /mcr, TZP  /mcr, TPON/mcr, &
                    TDON/mcr, TSiOH4/mcr, TOpal/mcr
      open( 11, file='Forcing.csv', form='FORMATTED' )
      write(11,'(A,13(",", A))') 'Time(day)','Lint0', 'TMP', 'MLD', 'ExcTime'
      write(11,'(A,13(",", 1PE10.4))') CTime, Lint0, Temp, MLD, ExcTime*d2s
!
!     ****** Main Loop *****
      do lt = 1, nt
!        ..... time control (Season : 0 to 1, parcentage in a year).....
         Tbefore = TTime                                 ! one step before present time
         TTime   = TTime + dt                            ! present time (real8 form)
         CTime = TT2CD(TTime)                            ! present time (charactor form)
         CALL TT2ND(Iyr, Imon, Iday ,Ihour, Imin, Isec ,TTime)
         Season = ( TTime                  - ND2TT(Iyr  ,1,1,0,0,0) )/  &
                  ( ND2TT(Iyr+1,1,1,0,0,0) - ND2TT(Iyr  ,1,1,0,0,0) )
!
!        ..... Example of Boundray condition .....
         Lint0 = 0.1d0 * ( 1.0D0 + 0.3d0 * cos( 2.0d0*3.1415926536d0*(Season - 0.50D0) ) )
         Temp  =           6.0D0 + 4.0d0 * cos( 2.0d0*3.1415926536d0*(Season - 0.65D0) )
         if (Temp .lt. 4.0 ) then
            MLD = MLD + dt * (150.0d0 - MLD ) / ( 100.0d0 * d2s )
            ExcTime = ExcTime + dt * ( 1.0d0/( 40.0d0*d2s) - ExcTime ) / (100.0d0*d2s)
         else
            MLD = MLD + dt * ( 30.0d0 - MLD ) / (   5.0d0 * d2s )
            ExcTime = ExcTime + dt * ( 1.0d0/(100.0d0*d2s) - ExcTime ) / (  5.0d0*d2s)
         end if
!
!        ..... Light Factors (LfcS, LfcL).....
         Lint = Lint0
         LfcDS = Lint/IoptS * exp(1.0D0 - Lint/IoptS)
         LfcDL = Lint/IoptL * exp(1.0D0 - Lint/IoptL)
         LfcS  = 0.0D0
         LfcL  = 0.0D0
         Kappa = alpha1 + alpha2 * ( TPS + TPL )
         dLint = exp( -Kappa * (MLD/LLN) )
         do L = 1, LLN
            LfcUS  = LfcDS
            LfcUL  = LfcDL
            Lint   = Lint       * dLint
            LfcDS  = Lint/IoptS * exp( 1.0D0 - Lint/IoptS )
            LfcDL  = Lint/IoptL * exp( 1.0D0 - Lint/IoptL )
            LfcS   = LfcS + ( LfcUS + LfcDS ) * 0.5D0 / LLN
            LfcL   = LfcL + ( LfcUL + LfcDL ) * 0.5D0 / LLN
         end do
!        ..... Photosynthesis of PS .....
         GppNPSn   = Mich( KNO3S, TNO3 ) * exp( - PusaiS * TNH4 )
         GppAPSn   = Mich( KNH4S, TNH4 )
         GppPSn    = Td(VmaxS, KGppS) * LfcS * TPS * ( GppNPSn + GppAPSn )
         RnewS     = GppNPSn / ( GppNPSn + GppAPSn )
         ResPSn    = Td( ResPS0, KResPS ) * TPS
         MorPSn    = Td( MorPS0, KMorPS ) * TPS * TPS
         ExcPSn    = GammaS               * GppPSn
!        ..... Photosynthesis of PL .....
         GppNPLn   = Mich( KNO3L, TNO3   ) * exp( - PusaiL * TNH4 )
         GppAPLn   = Mich( KNH4L, TNH4   )
         GppSiPLsi = Mich( KSiL , TSiOH4 )
         GppPLn    = Td(VmaxL, KGppL) * LfcL * TPL * min( ( GppNPLn + GppAPLn ), GppSiPLsi )
         RnewL     = GppNPLn / ( GppNPLn + GppAPLn )
         ResPLn    = Td( ResPL0, KResPL ) * TPL
         MorPLn    = Td( MorPL0, KMorPL ) * TPL * TPL
         ExcPLn    = GammaL               * GppPLn
!        ..... Grazing PS, PL, ZS, ZL --> ZS, ZL, ZP .....
         GraPS2ZSn = Td(GRmaxS,  KGraS) *GraF(LamS,PS2ZSstar,TPS) *TZS
         GraPS2ZLn = Td(GRmaxLps,KGraL) *GraF(LamL,PS2ZLstar,TPS) *TZL
         GraPL2ZLn = Td(GRmaxLpl,KGraL) *GraF(LamL,PL2ZLstar,TPL) *TZL
         GraZS2ZLn = Td(GRmaxLzs,KGraL) *GraF(LamL,ZS2ZLstar,TZS) *TZL
         GraPL2ZPn = Td(GRmaxPpl,KGraP) *GraF(LamP,PL2ZPstar,TPL) *TZP * exp( -PusaiPL *(TZL + TZS))
         GraZS2ZPn = Td(GRmaxPzs,KGraP) *GraF(LamP,ZS2ZPstar,TZS) *TZP * exp( -PusaiZS * TZL )
         GraZL2ZPn = Td(GRmaxPzl,KGraP) *GraF(LamP,ZL2ZPstar,TZL) *TZP
!        ..... Mortality, Excration, Egestion for Zooplanktons
!        ..... Commented out after Saito-san Meeting at 19 Jun, 2000 .....
!        BetaZS = 0.3 ** (  1.0 + Mich( TPL, TPS )  )
         ExcZSn = (AlphaZS-  BetaZS)   * GraPS2ZSn
         EgeZSn = (1.0    - AlphaZS)   * GraPS2ZSn
         MorZSn = Td( MorZS0, KMorZS ) * TZS * TZS
         ExcZLn = (AlphaZL-  BetaZL)   * (GraPS2ZLn+GraPL2ZLn+GraZS2ZLn)
         EgeZLn = (1.0    - AlphaZL)   * (GraPS2ZLn+GraPL2ZLn+GraZS2ZLn)
         MorZLn = Td( MorZL0, KMorZL ) * TZL * TZL
         ExcZPn = (AlphaZP-  BetaZP)   * (GraPL2ZPn+GraZS2ZPn+GraZL2ZPn)
         EgeZPn = (1.0    - AlphaZP)   * (GraPL2ZPn+GraZS2ZPn+GraZL2ZPn)
         MorZPn = Td( MorZP0, KMorZP ) * TZP * TZP
!        ..... Decomposition PON, DON, Opal ---> NH4, DON, SiOH4 .....
         DecP2N    = Td( VP2N0  , KP2N ) * TPON 
         DecP2D    = Td( VP2D0  , KP2D ) * TPON 
         DecD2N    = Td( VD2N0  , KD2N ) * TDON 
         DecO2S    = Td( VO2S0  , KO2S ) * TOpal
         Nit       = Td( Nit0   , KNit ) * TNH4 
!        ..... silica fluxes .....
         GppPLsi    = GppPLn    * RSiN
         ResPLsi    = ResPLn    * RSiN
         MorPLsi    = MorPLn    * RSiN
         ExcPLsi    = ExcPLn    * RSiN
         GraPL2ZLsi = GraPL2ZLn * RSiN
         GraPL2ZPsi = GraPL2ZPn * RSiN
         EgeZLsi    = GraPL2ZLsi
         EgeZPsi    = GraPL2ZPsi
!
!        ..... Tendency Terms for biological processes .....
         QNO3   = -( GppPSn - ResPSn ) * RnewS   &
                  -( GppPLn - ResPLn ) * RnewL + Nit
         QNH4   = -( GppPSn - ResPSn ) * (1.0 - RnewS)   &
                  -( GppPLn - ResPLn ) * (1.0 - RnewL)   &
                  - Nit + DecP2N + DecD2N + ExcZSn + ExcZLn + ExcZPn
         QPS    = GppPSn - ResPSn - MorPSn - ExcPSn - GraPS2ZSn - GraPS2ZLn
         QPL    = GppPLn - ResPLn - MorPLn - ExcPLn - GraPL2ZLn - GraPL2ZPn
         QZS    = GraPS2ZSn - GraZS2ZLn - MorZSn - ExcZSn - EgeZSn - GraZS2ZPn
         QZL    = GraPL2ZLn + GraZS2ZLn - MorZLn - ExcZLn - EgeZLn + GraPS2ZLn - GraZL2ZPn
         QZP    = GraPL2ZPn + GraZS2ZPn - MorZPn - ExcZPn - EgeZPn + GraZL2ZPn
         QPON   = MorPSn  + MorPLn  + MorZSn  + MorZLn + MorZPn   &
                + EgeZPn  + EgeZSn  + EgeZLn  - DecP2N - DecP2D
         QDON   = ExcPSn  + ExcPLn  + DecP2D  - DecD2N
         QSiOH4 =-GppPLsi + ResPLsi + ExcPLsi + DecO2S
         QOpal  = MorPLsi + EgeZLsi + EgeZPsi - DecO2S
!
!        ..... Exchange Fluxes between the Surface and Deep Layers .....
         ExpPON   = setVPON  / MLD * TPON
         ExpOpal  = setVOpal / MLD * TOpal
         ExcNO3   = ExcTime * ( TNO3d   - TNO3   )
         ExcSiOH4 = ExcTime * ( TSiOH4d - TSiOH4 )
         QNO3   = QNO3   + ExcNO3
         QSiOH4 = QSiOH4 + ExcSiOH4
         QPON   = QPON   - ExpPON
         QOpal  = QOpal  - ExpOpal
!
!        ...... Time Integration with Forward Scheme .....
         TNO3   = TNO3   + dt * QNO3  
         TNH4   = TNH4   + dt * QNH4  
         TPS    = TPS    + dt * QPS   
         TPL    = TPL    + dt * QPL   
         TZS    = TZS    + dt * QZS   
         TZL    = TZL    + dt * QZL   
         TZP    = TZP    + dt * QZP   
         TPON   = TPON   + dt * QPON  
         TDON   = TDON   + dt * QDON  
         TSiOH4 = TSiOH4 + dt * QSiOH4
         TOpal  = TOpal  + dt * QOpal
!
!        ..... Vertical Migration of ZL .....
         TZdwn = ND2TT(Iyr, ImonD, IdayD ,IhourD, IminD, IsecD )
         TZup  = ND2TT(Iyr, ImonU, IdayU ,IhourU, IminU, IsecU )
         if ( (Tbefore .lt. TZdwn).and.(TTime .ge. TZdwn) ) then
            TZLd = TZL
            TZL  = 0.0
            write(*,*) '*** Down ***', CTime
         end if
         if ( (Tbefore .lt. TZup).and.(TTime .ge. TZup) ) then
            TZL = SVRate * TZLd
            write(*,*) '***  UP  ***', CTime
         end if
!
!        ..... Monitor .....
         if ( int(TTime/Tmon).ne. int(Tbefore/Tmon) ) then
!           write(*,'(A,13(",", F8.4))') CTime, Season
            write(10,'(A,11(",", F8.4))') CTime, &
                    TNO3/mcr, TNH4  /mcr, TPS  /mcr, TPL /mcr, &
                    TZS /mcr, TZL   /mcr, TZP  /mcr, TPON/mcr, &
                    TDON/mcr, TSiOH4/mcr, TOpal/mcr
            write(11,'(A,13(",", 1PE10.4))') CTime, Lint0, Temp, MLD, ExcTime*d2s
         end if
      end do
!
      close(10); close(11)
!
      stop
      end
!*****************************************************************************************
!* Utilities for Date Control  Writtien by Yasuhiro Yamanaka (galapen@ees.hokudai.ac.jp) *
!*****************************************************************************************
!     exp. 1997/12/31 23:59:59 --> 6.223158719900000E+10
!     exp. 0000/01/01 00:00:00 --> 0.000000000000000E+00
!*****************************************************************************************
      real(8) function CD2TT( Cdate )
!
      integer       :: Iyr, Imon, Iday , Ihour, Imin, Isec
      real(8)       :: ND2TT
      character(19) :: Cdate
!
      if ( len( Cdate ) .ne. 19 ) then
         write(*,*) '### Length of date is no good ###'
         stop
      end if
      read (Cdate( 1: 4),*)  Iyr
      read (Cdate( 6: 7),*)  Imon
      read (Cdate( 9:10),*)  Iday
      read (Cdate(12:13),*)  Ihour
      read (Cdate(15:16),*)  Imin
      read (Cdate(18:19),*)  Isec
!
      CD2TT = ND2TT(Iyr, Imon, Iday , Ihour, Imin, Isec)
!
      return
      end function
!***********************************************************************
!     exp. 6.223158719900000E+10 --> 1997/12/31 23:59:59
!***********************************************************************
      character(19) function TT2CD(tt)
!
      integer :: Iyr, Imon, Iday , Ihour, Imin, Isec
      real(8) :: tt
!
      call TT2ND( Iyr, Imon, Iday, Ihour, Imin, Isec , tt )
!
      write(TT2CD,'(I4.4,5(A,I2.2))') Iyr, '/', Imon, '/', Iday, &
                               ' ', Ihour, ':', Imin, ':', Isec
!
      return
      end function
!***********************************************************************
!     exp. 1997,12,31,23,59,59 --> 6.223158719900000E+10
!***********************************************************************
      real(8) function ND2TT(Iyr, Imon, Iday, Ihour, Imin, Isec)
!
      integer   :: IM2D(12,0:1) = &
         reshape( (/ 0,31,59,90,120,151,181,212,243,273,304,334,  &
                     0,31,60,91,121,152,182,213,244,274,305,335  /), (/12,2/) )
      integer   :: Iyr, Imon, Iday, Ihour, Imin, Isec
      integer   :: Iy4, Iy1, Ileap, Im, Itt
!
!
      Iy4 = 1461 * ( Iyr / 4 )
      Iy1 = 365 * mod( Iyr, 4 )
!
      if ( mod( Iyr, 4 ) .ne. 0 ) then
         Ileap = 0
      else
         Ileap = 1
      end if
      Im = IM2D( Imon, Ileap)
!
      Itt = Iy4 + Iy1 + Im + Iday - Ileap
!
      ND2TT = Ihour * 3600 + Imin * 60 + Isec
      ND2TT = ND2TT + Itt * 86400.0D0
!
      return
      end function
!***********************************************************************
!     exp. 6.223158719900000E+10 --> 1997,12,31,23,59,59
!***********************************************************************
      subroutine TT2ND(                                        &
                 Iyr   , Imon  , Iday   , Ihour, Imin, Isec,   & !O & I
                 tt    )
!
      integer :: Iyr, Imon, Iday , Ihour, Imin, Isec
      integer :: Itt, Iy, Iy4, Iyd, Iy1, Ileap, Imd, Im, Its
      integer   :: IM2D(12,0:1) = &
         reshape( (/ 0,31,59,90,120,151,181,212,243,273,304,334,  &
                     0,31,60,91,121,152,182,213,244,274,305,335  /), (/12,2/) )
      integer :: IY2D(4) = (/0,366,731,1096/)
      real(8) :: tt, tt0, ND2TT
!
!
!     ..... ITT [day] .....
      Itt = 1 + tt / 86400.0D0
!
      Iy4   = (Itt-1) / 1461
      Iyd   = Itt - Iy4 * 1461
      do IY = 1, 4
         if ( IY2D(Iy) + 1 .le. Iyd ) then
            Iy1 = Iy
         end if
      end do
!
      Iyr   = Iy4 * 4 + Iy1 - 1
      if ( mod(Iyr,4) .ne. 0 ) then
         Ileap = 0
      else
         Ileap = 1
      end if
      IMD = IYD - IY2D(IY1)
!
      do IM = 1, 12
         if ( IM2D(IM,ILEAP)+1 .le. IMD ) then
            IMON = IM
         end if
      end do
      IDAY = IMD - IM2D(IMON,ILEAP)
!
      TT0 = ND2TT(IYR, IMON, IDAY ,0,0,0)
      ITS = nint(  TT - TT0 )
      Ihour = ITS / 3600
      Imin  = ( ITS - Ihour * 3600 ) / 60
      Isec  = ITS - Ihour * 3600 - Imin * 60
!
      return
      end subroutine
